'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var t = _ref.types;

	var classVisitor = function classVisitor(path, _ref2) {
		var _ref2$opts$decoratorN = _ref2.opts.decoratorNames,
		    decoratorNames = _ref2$opts$decoratorN === undefined ? [] : _ref2$opts$decoratorN;

		if (!path.node.decorators) {
			return;
		}

		var usedDecoratorNames = path.node.decorators.filter(function (decorator) {
			return decorator.expression.callee;
		}).map(function (decorator) {
			return decorator.expression.callee.name;
		});

		var hasDecorator = usedDecoratorNames.some(function (name) {
			return decoratorNames.includes(name);
		});

		if (!hasDecorator) {
			return;
		}

		var ctorNode = getCtorNode(path.node);

		if (!ctorNode) {
			return;
		}

		var paramNames = getMethodParamNames(ctorNode, t);

		if (!paramNames.length) {
			return;
		}

		// Create `$inject` property, e.g. `Foo.$inject = ['bar', 'baz];`
		var injectExp = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(path.node.id.name), t.identifier('$inject')), t.arrayExpression(paramNames.map(function (name) {
			return t.stringLiteral(name);
		}))));

		// append to parent node if class is "default-exported"
		if (path.parentPath.type === 'ExportDefaultDeclaration') {
			path.parentPath.insertAfter(injectExp);
		} else {
			path.insertAfter(injectExp);
		}
	};

	return {
		visitor: {
			ClassDeclaration: classVisitor
		}
	};
};

/**
 * Finds a constructor method in class body.
 * @param {Object} node Class body nodes.
 * @return {Object} Constructor method node.
 */
function getCtorNode(node) {
	return node.body.body.find(function (def) {
		return def.kind === 'constructor';
	});
}

/**
 * Returns a list of method parameter names.
 * @param {Object} node Node to get parameter names from.
 * @param {Object} babel.types babel-types
 */
function getMethodParamNames(node, t) {
	return node.params
	// ignore rest parameter
	.filter(function (param) {
		return t.isIdentifier(param);
	}).map(function (param) {
		return param.name;
	});
}

/**
 * Babel plugin to automatically annotate functions for Angular dependency injection.
 * @param {Object} babel The current babel object.
 * @param {Object} babel.types babel-types
 * @return {Object} Babel visitor.
 */